--
-- luako-zrhook.lua
--

---- kolabels-utf.tex ̃_~[

if not kpse.find_file("kolabels-utf.tex") then
  local f = io.open("kolabels-utf.tex", "w")
  f:write("\\relax\n"); f:close()
end

---- nodes W[̌݊΍

local zr_nodes = {}
do
  local nodes = zr_nodes

  -- Excerpts from node-res.lua, in ConTeXt
  local copy_node, new_node = node.copy, node.new
  local reserved = { }
  local function register_node(n)
      reserved[#reserved+1] = n
      return n
  end
  local kern              = register_node(new_node("kern",1))
  local penalty           = register_node(new_node("penalty"))
  local glue              = register_node(new_node("glue"))
  local glue_spec         = register_node(new_node("glue_spec"))
  local rule              = register_node(new_node("rule"))
  function nodes.penalty(p)
      local n = copy_node(penalty)
      n.penalty = p
      return n
  end
  function nodes.glue_spec(width,stretch,shrink)
      local s = copy_node(glue_spec)
      s.width, s.stretch, s.shrink = width, stretch, shrink
      return s
  end
  local function someskip(skip,width,stretch,shrink)
      local n = copy_node(skip)
      if not width then
          -- no spec
      elseif tonumber(width) then
          local s = copy_node(glue_spec)
          s.width, s.stretch, s.shrink = width, stretch, shrink
          n.spec = s
      else
          -- shared
          n.spec = copy_node(width)
      end
      return n
  end
  function nodes.glue(width,stretch,shrink)
      return someskip(glue,width,stretch,shrink)
  end
  function nodes.rule(width,height,depth,dir)
      local n = copy_node(rule)
      if width  then n.width  = width  end
      if height then n.height = height end
      if depth  then n.depth  = depth  end
      if dir    then n.dir    = dir    end
      return n
  end

end

local luatex_fonts = "luatex-fonts.lua"
local lua_dofile = dofile
function dofile(fname)
  if fname:sub(-luatex_fonts:len()) == luatex_fonts then
    lua_dofile(fname); patch_to_lib_nodes()
    _G.dofile = lua_dofile
  else
    return lua_dofile(fname)
  end
end

function patch_to_lib_nodes()
  local t = {"penalty", "glue", "rule"}
  for _, v in pairs(t) do
    if not nodes[v] then nodes[v] = zr_nodes[v] end
  end
end

-- EOF
